#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Yufei,2023
# github.com/Yflyer
import pickle,os,time,subprocess
import pandas as pd
import shutil
from Bio import SeqIO

######################################################################################
### Input parameters of script ###
input_dir = 'pe_data'
dict_path = 'pe_dict.pkl'
blast_dir = 'blast'

Ref_path = '../Ref/Ecoil/Ecoil'
#Ref_path = '../Ref/mmseqs_80c/mmseqs_80c'

def mkdir(dir):
	if not os.path.exists(dir):
		os.makedirs(dir)
		print ("--- Create a new directory ---")
	else:
		print ("--- The directory has already existed! ---")
mkdir(blast_dir)

# Create an empty DataFrame to store the mean values
mean_df = pd.DataFrame(columns=['Run_ID','Lib_type', 'Mean_sstart', 'Mean_send','Match_ratio'])

# Import the dictionary from the file using pickle
with open(dict_path, 'rb') as f:
    seq_dict = pickle.load(f)

mapped_num = rev_num = unmapped_num = 0
for data_id in seq_dict:
    
    ######################################################################################
    # Run BLAST and count the number of bacterial hits
    filename = seq_dict[data_id]
    blast_output = data_id + "_blast.tsv"

    print('------------------------------------------------------')
    print('------{}: now start blast file {}'.format(time.strftime("%Y-%m-%d %X"),filename))
    
    cmd = f"seqkit sample -n 100 {input_dir}/{filename} > {blast_dir}/{data_id}_100.fasta"
    subprocess.run(cmd,stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True )
    cmd = f"blastn -query {blast_dir}/{data_id}_100.fasta -db {Ref_path} -outfmt '6 qseqid length qstart qend sseqid sstart send evalue bitscore'  -num_alignments 1 -out {blast_dir}/{blast_output} -num_threads 12"
    subprocess.run(cmd,stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True )

    ######################################################################################
    # Check the mapped status of each data
    if os.path.getsize(f"{blast_dir}/{blast_output}") != 0:
        df = pd.read_csv(f"{blast_dir}/{blast_output}",sep='\t', header=None) 
        Match_ratio = round(df.shape[0]/100,3)
        Mean_length,Mean_qstart,Mean_qend,Mean_sstart,Mean_send=round(df.iloc[:,[1,2,3,5,6]].mean(),0)
                                                
        # check requirement of map length  
        if Mean_length > 150:
            print(f"{data_id} (query position:{Mean_qstart}-{Mean_qend}) matched at the ref position from {Mean_sstart} to {Mean_send} (length: {Mean_length})")
            data_stat = pd.DataFrame({'Run_ID': data_id, 'Lib_type': input_dir,'Mean_sstart': Mean_sstart, 'Mean_send': Mean_send,'Match_ratio':Match_ratio}, index=[data_id])
            mapped_num += 1
        else:
            print(f"{data_id} can not match to reference sequences")
            Mean_qstart = Mean_qend = 0
            data_stat = pd.DataFrame({'Run_ID': data_id, 'Lib_type': input_dir,'Mean_sstart': 0, 'Mean_send': 0,'Match_ratio':0}, index=[data_id])
            unmapped_num += 1

    else:
        print(f"{data_id} can not match to reference sequences")
        Mean_qstart = Mean_qend = 0
        data_stat = pd.DataFrame({'Run_ID': data_id, 'Lib_type': input_dir,'Mean_sstart': 0, 'Mean_send': 0,'Match_ratio':0}, index=[data_id])
        unmapped_num += 1

    mean_df = pd.concat([mean_df, data_stat], ignore_index=True)

    ######################################################################################
    # check reverse between R1 and R2
    if  Mean_qstart>Mean_qend:
        print(f'* R1 and R2 files are exchange in {data_id}. Now reverse merged files to forward orientation.')
        rev_num += 1
        
        Input_fasta = SeqIO.parse(f"{input_dir}/{filename}", "fasta")

        # Create a list of records to write to the output file
        new_records = []

        # Reverse complement the sequence and update the record
        for record in Input_fasta:
            record.seq = record.seq[::-1]
            new_records.append(record)

        with open(f"{input_dir}/Rev_{filename}", "w") as Rev_fasta: 
             SeqIO.write(new_records, Rev_fasta, "fasta")
        
        #shutil.move(f"{input_dir}/Rev_{filename}", f"{input_dir}/{filename}")

    print('------------------------------------------------------')

mean_df.to_csv(f'{input_dir}_Ecoilmap_summary.csv', index=False)


print('----------------------------------------------------')
print('----------------------Summary-----------------------')
print(f'In {len(seq_dict)} data of {input_dir}:')
print(f'{mapped_num} data are mapped to reference sequences.')
print(f'{rev_num} data are mapped but reverse.')
print(f'{unmapped_num} data can not be mapped.')
print('----------------------------------------------------')
            





#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Yufei,2023
# github.com/Yflyer
import os,subprocess,pickle,re
from Bio import SeqIO

######################################################################################
### Input parameters of script ###
raw_dir = 'raw_data'
se_dir = 'se_data'
pe_dir = 'pe_data'

def mkdir(dir):
	if not os.path.exists(dir):
		os.makedirs(dir)
		print ("--- Create a new directory ---")
	else:
		print ("--- The directory has already existed! ---")
mkdir(pe_dir)

# create a dictionary to store the classification information
raw_se_dict = {}
raw_pe_dict = {}

######################################################################################
# index types of library
for file in os.listdir(raw_dir):

    # check if the file ends with .fastq
    if file.endswith('_pass.fastq'):
        # extract the data ID from the file name
        data_id = re.split(r'[^a-zA-Z0-9]+', file)[0]

        # count sequence for the file
        cmd=f"cat {raw_dir}/{file} |  grep \"^@\" | wc -l"
        read_count = int(subprocess.getoutput(cmd))

        # classify the file as single_end
        if read_count>1000:
            raw_se_dict[data_id] = file
            print("### single-end dataset constructed:",data_id,":",raw_se_dict[data_id])
        else:
            print(file,"have not-enough sequence to following work!")

    # check if the file name contains '_1' or '_2'
    if '_1' in file or '_2' in file:
        # extract the data ID from the file name
        data_id = re.split(r'[^a-zA-Z0-9]+', file)[0]

        # count sequence for the file
        cmd=f"cat {raw_dir}/{file} |  grep \"^@\" | wc -l"
        read_count = int(subprocess.getoutput(cmd))

        # select the file for pair_end
        if read_count>1000:
            raw_pe_dict.setdefault(data_id, []).append(file)
            raw_pe_dict[data_id].sort()
        else:
            print(file,"have not-enough sequence to following work!")

######################################################################################
### sort out pair_end sequences
for data_id in list(raw_pe_dict.keys()):
    # sort out non-pair files as single-end file
    if len(raw_pe_dict[data_id]) == 1:
        raw_se_dict[data_id] = raw_pe_dict[data_id][0]
        raw_pe_dict.pop(data_id)
        print(f'{raw_se_dict[data_id]} was the only used file in {data_id} pair files and regarded as single-end dict')
    else:
        ### output the well paired sequences and unpaired sequences to pe_dir
        fwd_seq, rev_seq = raw_pe_dict[data_id]

        cmd = f"seqkit pair --id-regexp '^(\S+)\.\s?' -1 {raw_dir}/{fwd_seq} -2 {raw_dir}/{rev_seq} -O {pe_dir} -t dna -u"
        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        #os.remove(f"{raw_dir}/{fwd_seq}")
        #os.remove(f"{raw_dir}/{rev_seq}")

        print('--- sorting out paired sequences from:',data_id, ' ---')
        print("### pair-end dataset constructed:",data_id,":",raw_pe_dict[data_id])
        
# Import the dictionary from the file using pickle
with open('raw_se_dict.pkl', 'wb') as f:
    pickle.dump(raw_se_dict, f)

with open('raw_pe_dict.pkl', 'wb') as f:
    pickle.dump(raw_pe_dict, f)


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here,"README.md"),encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.1'
DESCRIPTION = 'Processing open-source amplicon database for ecology'
LONG_DESCRIPTION = 'A package that allows to download, quality control, file error-correction and match of open-source amplicon data'

setup(
    name='madt',
    version=VERSION,
    packages=find_packages(),
    author="Yufei Zeng",
    author_email="yfzeng0827@hotmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    entry_points={
        'console_scripts': [
            'madt_DataIndex = madt.madt_DataIndex:main',
            'madt_LibCheck = madt.madt_LibCheck:main',
            'madt_LipMap = madt.madt_LipMap:main'
        ]
    },
    keywords=['python', 'bioinformatic', 'amplicon', 'ecology'],
    install_requires=[
        'biopython',
        'pandas',
        'seqkit',
        'bbmap'
    ]
)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Yufei,2023
# github.com/Yflyer

from Bio import SeqIO
import os,subprocess,re,pickle,shutil


######################################################################################
### Input parameters of script ###
raw_dir = 'raw_data'
se_dir = 'se_data'
pe_dir = 'pe_data'
up_dir = 'up_data'
raw_se_path = 'raw_se_dict.pkl'
raw_pe_path = 'pe_dict.pkl'

pe_dict = {}
se_dict = {}
up_dict = {}
um_dict = {}

def mkdir(dir):
	if not os.path.exists(dir):
		os.makedirs(dir)
		print ("--- Create a new directory ---")
	else:
		print ("--- The directory has already existed! ---")
mkdir(up_dir)
mkdir(se_dir)
mkdir(pe_dir)

######################################################################################
### index pair_end dataset
for file in os.listdir(pe_dir):
    # check if the file ends with .fastq
    if file.endswith('.fastq'):
        # extract the data ID from the file name
        data_id = re.split(r'[^a-zA-Z0-9]+', file)[0]

        # check if the file name contains '_1' or '_2'
        if '_1.fastq' in file or '_2.fastq' in file:
            # classify the file as potential_pair
            pe_dict.setdefault(data_id, []).append(file)
            pe_dict[data_id].sort()
        elif 'unpaired_1.fastq' in file or 'unpaired_2.fastq' in file:
            # classify the file as up_dict
            up_dict.setdefault(data_id, []).append(file)
            up_dict[data_id].sort()

            # move unpaired files
            if len(up_dict[data_id]) ==2: 
                shutil.move(f"{pe_dir}/{up_dict[data_id][0]}", f"{up_dir}/{up_dict[data_id][0]}")
                shutil.move(f"{pe_dir}/{up_dict[data_id][1]}", f"{up_dir}/{up_dict[data_id][1]}")
                print(f'* Unpaired sequences have been moved to {up_dir} for {data_id}: {up_dict[data_id]}')

#######################################################################################
# PE check
for data_id in list(pe_dict.keys()):
    ####################################################################################### 
    # check duplication
    print('### Now Check pairs from ',data_id)
    fastq1 = SeqIO.parse(f"{pe_dir}/{pe_dict[data_id][0]}", "fastq")
    fastq2 = SeqIO.parse(f"{pe_dir}/{pe_dict[data_id][1]}", "fastq")
    
    # We check the first 20 sequences to determine whether a duplication.
    seq1 = seq2 = ''
    check20=0
    for record1, record2 in zip(fastq1, fastq2):
        # Extract the sequence from the two records
        seq1 += str(record1.seq)
        seq2 += str(record2.seq)
        check20 += 1 
        if check20 == 20: break

    # sort out duplications and merge other pair-end
    if seq1 == seq2:
        # move duplications to se and remove from pair dict
        cmd = f"bbduk.sh in={pe_dir}/{pe_dict[data_id][0]} out={se_dir}/{data_id}_Dup.fasta qtrim=rl trimq=20 minlen=150"
        result = subprocess.run(cmd,stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True )
        print(f'------Duplication sequences have been moved to {se_dir} of {pe_dict[data_id][0]} and cleaned as Single-End data: {data_id}_SE.fasta')
        os.remove(f"{pe_dir}/{pe_dict[data_id][0]}")
        os.remove(f"{pe_dir}/{pe_dict[data_id][1]}")
        pe_dict.pop(data_id)
    else:
        # Try to merge
        fwd_seq = pe_dict[data_id][0]
        rev_seq = pe_dict[data_id][1]
        cmd = f"bbmerge.sh in1={pe_dir}/{fwd_seq} in2={pe_dir}/{rev_seq} out={pe_dir}/{data_id}_merged.fastq"
        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        
        # count sequence for the merged file
        cmd=f"cat {pe_dir}/{data_id}_merged.fastq |  grep \"^@\" | wc -l"
        merge_count = int(subprocess.getoutput(cmd))
        cmd=f"cat {pe_dir}/{fwd_seq} |  grep \"^@\" | wc -l"
        raw_count = int(subprocess.getoutput(cmd))

        # filter the merged files
        if merge_count/raw_count<0.1:
            os.remove(f"{pe_dir}/{fwd_seq}")
            os.remove(f"{pe_dir}/{rev_seq}")
            os.remove(f"{pe_dir}/{data_id}_merged.fastq")
            pe_dict.pop(data_id)

            um_dict[data_id] = [fwd_seq,rev_seq]
            print('* Unmerged ',data_id, ' (Merged rate: ',round(merge_count/raw_count,3),') was removed')
            
        else:
            os.remove(f"{pe_dir}/{fwd_seq}")
            os.remove(f"{pe_dir}/{rev_seq}")

            # PE QC
            cmd = f"bbduk.sh in={pe_dir}/{data_id}_merged.fastq out={pe_dir}/{data_id}_PE.fasta qtrim=rl trimq=20 minlen=150"
            result = subprocess.run(cmd,stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True )
            os.remove(f"{pe_dir}/{data_id}_merged.fastq")

            pe_dict[data_id]=f"{data_id}_PE.fasta"
            print(f'------Merged pair-end sequences of {pe_dict[data_id]} have been cleaned: {pe_dict[data_id]}')

#######################################################################################
# SE check
with open('raw_se_dict.pkl', 'rb') as f:
    raw_se_dict = pickle.load(f)

for data_id in list(raw_se_dict.keys()):
    cmd = f"bbduk.sh in={raw_dir}/{raw_se_dict[data_id]} out={se_dir}/{data_id}_SE.fasta qtrim=rl trimq=20 minlen=150"
    result = subprocess.run(cmd,stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True )
    print(f'------Single-end sequences of {raw_se_dict[data_id]} have been cleaned: {data_id}_SE.fasta')

f.close()




